
# q-expansion of newform 616.4.a.g, downloaded from the LMFDB on 14 January 2025.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 616
weight = 4
poly_data = [-2576, 1858, 92, -105, -2, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0, 0, 0], 1], [[0, 1, 0, 0, 0], 1], [[978, 61, -363, -13, 5], 257], [[7156, -3395, -496, 41, 4], 257], [[-12493, -636, 983, 26, -10], 257]]

hecke_ring_character_values = None
aps_data = [[0, 0, 0, 0, 0], [0, 1, 0, 0, 0], [-2, -1, -1, 0, 0], [-7, 0, 0, 0, 0], [-11, 0, 0, 0, 0], [39, -1, 1, 1, -1], [-47, -3, -2, 2, -3], [-38, -2, 3, 1, -2], [-7, 0, 1, -3, -1], [-61, -21, 5, -8, 3], [-32, -9, 1, 5, 4], [81, -28, 2, -6, 7], [-166, 0, -11, 4, 0], [-217, 35, -2, -1, 3], [-26, -22, -7, -2, -6], [-182, -18, 3, -5, 6], [-134, 65, -4, -14, 6], [183, -27, -5, 1, -13], [-369, -24, 29, -11, 5], [83, 16, -25, -1, 25], [-34, 52, 3, -10, -8], [-18, 50, 3, -3, -24], [-67, -59, 32, 3, 23], [-78, -69, 6, -23, -8], [-236, -93, 10, -37, 10], [-396, 70, -24, 25, -14], [-862, 6, 21, 20, 6], [-377, 103, 29, 18, -3], [99, 69, 61, 2, -9], [479, 70, 49, -23, 19], [-514, -40, -98, 6, -30], [-347, 107, -28, 15, -29], [501, 270, 54, -20, -25], [-33, 49, 54, 25, -7], [293, -123, -53, 6, -3], [-1250, -22, 31, 1, 40], [414, 51, -35, 46, 8], [634, -74, 118, 54, 74], [-296, 34, 52, -46, 60], [593, 97, 69, -11, 9], [311, 60, -49, -23, -15], [-88, -221, -81, 42, -30], [-761, 72, 77, -37, 61], [1301, -183, 3, -26, 5], [1068, -358, -100, -82, 86], [84, 248, 35, 64, 60], [-497, 15, 49, -58, -27], [-1172, 11, -83, 101, 104], [-666, -208, 69, 67, -166], [1690, 101, -195, 18, -24], [-183, -243, -139, -22, 33], [729, -445, -23, 34, 79], [-1208, -40, 171, -42, -94], [-2754, 377, 84, 76, -86], [120, 254, -104, 42, -82], [1031, 199, -1, 128, 25], [1376, 376, 63, -121, 102], [2005, -221, 59, -130, 11], [-2415, 683, 101, -126, 109], [-41, 97, -143, -62, 7], [-942, 8, -187, -65, -66], [-228, 44, 88, 27, 134], [-2161, -161, -222, -75, -55], [-762, -110, -193, -178, 146], [1684, 857, -96, -91, 2], [-487, 140, -257, 281, -191], [3057, -1016, -149, -111, 131], [2651, -209, 249, 164, -133], [68, -372, -75, 161, -14], [-799, -337, -173, 211, -263], [-3954, 15, -126, 169, -180], [-3507, -155, 40, -73, 141], [680, -17, -3, 75, -172], [3108, -136, -446, 122, -54], [1753, 288, -301, 199, 27], [-4108, -385, -271, 59, -152], [-1185, 90, 79, -113, 203], [3656, 88, 51, -223, 306], [-3952, 34, -30, 60, -146], [-1276, -18, -337, 204, -342], [-2184, -510, 348, -439, 164], [-3204, 268, -145, 333, -312], [-9274, -188, -121, -81, 64], [-4690, -251, 452, -173, 192], [-3825, 43, 227, -38, -323], [1685, -870, 443, 91, -1], [-3719, -44, 134, -2, 235], [1928, -780, -700, 12, -138], [-10879, 227, 197, -41, 133], [4071, 228, 249, -7, -95], [5056, -217, -160, 132, 48], [945, 651, 335, -38, -157], [3737, -452, 95, 367, -45], [1705, 711, -44, 497, -171], [6722, 462, -830, 90, -206], [-3655, 359, 303, 194, 199], [-3160, 2199, 395, 238, -294], [4534, -227, -68, 391, 16], [5823, -1713, -880, -17, -187], [2409, 963, 169, -182, 385], [5370, 862, 206, -176, 98], [-12845, 1243, 229, 330, 267], [-1400, -1222, -169, -43, 96], [-2575, 1721, 51, -424, 181], [-1088, 1020, 532, 388, -204], [5522, -1403, 14, -327, -332], [8182, -1456, -1192, 5, -426], [-8213, 443, 662, 184, -289], [-4330, 138, -508, 12, 494], [-4186, 130, -769, 492, -84], [-6650, -1368, -432, -80, 358], [-2060, 1256, -442, 284, -526], [-2540, -936, -325, -283, -436], [-10718, -81, -286, 254, 350], [1705, -1972, -631, 27, -73], [2395, 1490, 556, 150, 85], [4052, -1685, -446, 212, -4], [1744, -283, 605, 227, 316], [5945, 362, -97, 99, 261], [-3476, 1492, -607, 57, 2], [12358, -689, -263, -314, 108], [-1435, 395, 319, 22, 217], [2187, -963, 817, 427, -461], [-2150, -742, 74, 220, -566], [4118, 293, -164, -160, -246], [11112, -1572, 50, -396, 90], [1923, 992, 497, 47, -509], [17710, -1519, -601, -365, -362], [5690, -551, 347, -347, 754], [4652, 790, 680, 69, -234], [-24276, 166, 440, -238, 72], [-5246, -1442, -1034, 384, -626], [3753, 1510, 655, -315, -269], [-8916, -334, -94, 544, 518], [-4635, -1377, -40, -48, 697], [-1013, -2855, -276, -208, -17], [-2344, -340, -181, -313, 498], [16929, -125, -952, -215, -381], [-8424, -1431, -741, -542, -146], [-3402, 304, -1464, -48, -140], [-9146, -1194, -237, -233, 206], [-6212, -802, 1150, -596, -234], [2013, -2060, 293, 203, -293], [-1307, 861, -1418, 277, -963], [15818, -1345, 595, 908, -412], [-7648, -1305, -155, 123, -252], [24460, 1276, 610, 221, -62], [-5098, 418, 1517, -302, 88], [7686, -1051, -864, -16, -758], [-7248, 1816, -758, 630, 416], [20748, -3066, -1432, -236, 42], [-6462, 377, 2074, -11, -452], [13704, 1924, 1080, -944, 700], [-9278, -1016, -1444, -454, -134], [14228, -1700, 542, 656, -4], [-674, -3210, -1158, 562, -594], [-10099, 947, -459, 30, -221], [-11102, -272, -1166, -488, 52], [-5769, -1171, 682, 282, -77], [13850, 3350, -922, 683, -200], [-3477, -932, 1399, -1087, 525], [-18238, -1378, 1030, -1452, 596], [3661, -897, 1532, 49, 205], [3786, 1827, -906, 230, -642], [-15677, 906, -753, 147, 343], [-10574, -2655, 577, -1107, 114], [-20578, 2582, 22, -114, 202], [6192, 578, 870, 173, -710], [8467, 4717, 99, 326, -597], [-1454, -387, -677, -212, 308], [5989, 2839, 1575, -480, -105], [9772, -1192, -587, 179, 610], [-16934, 1022, -1905, -571, -1284], [18299, -1615, -831, 2, 715], [-7696, 3685, 1763, 981, 44], [-5382, 1873, 392, 1111, -216], [22909, 1887, 914, 755, -587], [-6225, 474, -968, 122, 169], [-28319, -1355, 285, -1058, 227], [-11423, -181, 1525, 377, -59], [3044, -376, -644, -828, 1356], [10779, -1769, 882, 427, 789], [22807, 2386, 549, -687, -193], [-8469, 1861, 1208, -350, -57], [-13303, 2106, 47, -213, 1087], [-14990, 182, 377, -1053, 356], [11699, -219, 1381, -564, -413], [-5388, -3430, -280, -1735, 952], [-10342, -4354, 138, 926, -408], [-9500, 2722, -154, 1544, -460], [-30798, 717, -512, 931, -1768], [-13691, 1997, 2571, -6, 1551], [-29237, -428, 777, -31, -647], [-11840, 6532, 1289, 43, 598], [-29458, 1390, 865, -573, 504], [-28344, -2519, -2248, 925, 10], [10764, 1252, -1076, -354, 1478], [-14763, 1804, -2, -522, 907], [-21448, 5638, 1879, 1242, -422], [-10729, -6705, 405, 812, -295], [26342, 3676, 560, 636, -636], [21429, 1023, -1685, 360, -957], [770, 534, -679, -73, 1264], [1227, 1755, 2052, -1672, 267], [7600, -3447, 492, -72, -216], [7638, -7349, -1547, -362, 140], [15750, -933, -1971, 923, -394], [4279, -3641, -1209, 396, 133], [-32808, 2820, 850, -974, 842], [20190, 2172, 3026, -1389, 750], [11206, 2122, -2824, -584, 1264], [-7768, -7741, 1323, 406, 494], [-17023, -2572, 567, 109, -457], [3178, 3260, 516, -919, 930], [-18630, -1778, -296, -464, 54], [10902, 5039, -2230, 473, -704], [-22951, 4157, 972, -641, -383], [-607, 673, 2938, -682, -567], [-24244, 3432, 546, 1672, -880], [-10293, -6136, -99, 667, 867], [-41305, -3055, -321, 972, 787], [-70780, -614, -476, 448, -16], [8785, 3174, 68, 978, -1049], [23769, -3990, 1237, 193, 1723], [18659, 8083, 1824, 587, -1015], [-17220, 1294, 452, -88, -1110], [-42962, 5068, 1168, 602, 1600], [-10618, 5026, 2754, -1080, 2002], [-897, 6877, -2081, -104, -1891], [18857, 2977, -220, 1387, 1433], [7595, -2848, -3470, 946, -1063], [-27992, -1204, -1421, -23, -68], [-56063, -1253, -223, -418, -137], [20816, -7900, 282, -1118, -990], [-5740, 439, 2624, -318, 164], [17910, -2244, 2154, -892, 1022], [-39464, -4553, -1018, 955, 998], [-2244, 4302, 5240, -74, 962], [-939, 2416, -453, 663, -681], [75736, -4579, -617, -337, 132], [21045, 6116, -1739, -791, -1517], [-6941, 901, -752, -1129, 765], [-27658, 3370, -1447, 1256, -454], [4845, 4214, -836, -226, 1411], [-2619, 1337, -2586, -336, 1405], [-9215, -4571, 3149, -68, -1749], [-7054, 9282, -903, -1328, -1098], [37030, -3228, 520, -629, -34], [-47890, 1152, -2318, 558, -778], [32986, -3760, -3518, -38, -898], [-17976, 10552, 1042, 1118, -2122], [-9950, -5146, -1, -1868, 1052], [4373, 3226, 1683, -83, 43], [43822, -6380, -1206, 914, -336], [25330, -7352, 214, -1263, 1648], [-32967, 2629, -1757, -1474, 195], [24850, -562, 1841, 2389, -1138], [17963, 1325, -2312, 3263, -2047], [35626, 2855, -1235, -626, -128], [18298, 1502, -1063, 70, -896], [9408, 2606, 1175, 1021, -1550], [55811, 2225, 555, -494, 609], [-4919, -6189, -2993, -1447, 1509], [35674, 2388, -4404, 1429, -1072], [-10822, -3332, -1512, 444, -652], [21930, 1476, -848, 665, 738], [-46447, -1536, -2258, 1414, 827], [-4950, 4701, -1256, 1903, -744], [2852, 1444, -743, -347, 2510], [21869, 4831, -271, 640, 93], [-24010, -2013, 3933, -186, 2296], [38821, 5356, -3919, 1245, -197], [-47010, 4072, 90, 668, -1084], [-20997, 8629, -521, -194, 1493], [25974, -2620, -4505, 2090, -1520], [-8144, -6061, -1759, -2191, 772], [4119, -410, 2401, 1325, 389], [20500, 3718, -334, -708, 1162], [61852, -4734, -2584, -666, -942], [26373, 7165, 5700, 1067, -1417], [-11361, 6009, 1995, -1254, 809], [-40413, 9180, -381, -1805, -271], [44256, 2566, -2816, 500, 192], [77452, -1876, 2346, 711, 918], [-3339, -6533, -4588, 217, -105], [-44909, 1204, -4339, -543, -735], [11948, 1592, 4395, 1081, 1168], [-76445, 2431, 697, -186, -633], [34942, -8773, 1001, 1661, -2242], [-29696, -8064, -3901, 2692, 70], [-25883, 1570, 764, 578, -1765], [20556, -978, -520, 3, -1440], [36836, 9564, 394, -1476, 1362], [25668, 8786, 3073, -401, -1572], [-15813, -1497, -2823, -1902, 895], [7131, 3203, -4593, -635, -1213], [-27496, -3055, 3839, 381, -196], [14296, 2739, 4835, -1602, -614], [-51163, 11207, 1342, 1599, -2657], [46795, -5661, -292, -1327, 1617], [4582, 5254, -1773, -1682, -1708], [51755, 9345, -2539, -150, -737], [-685, -7139, -1956, -2397, 2539], [-87140, 8100, 232, 2218, -1552], [-28077, 13146, 3789, 1415, -1407], [16421, 1020, 640, 1204, 3923], [12229, -88, 1865, -2245, -1173], [-2050, -4737, 1413, -1326, -3108], [14904, -6168, -4952, -972, -64], [-26193, -7703, -3295, -1800, 2131], [16609, 421, 2959, 2742, -1341], [-12206, -4752, -1224, 2328, 2552], [42384, -4590, -1044, 80, 2074], [-29749, -6104, 97, -3437, 849], [63269, 2187, 2836, -3731, 677], [-78811, -7315, 3576, -1756, -291], [-27968, 425, 202, -2292, -2608], [-5210, -13284, 2246, 1370, -2014], [21872, 2094, 2064, -468, 1210], [-90054, -5908, -1662, -236, 648], [-70941, -293, 3100, 1035, -1459], [28609, 1408, -1995, 2355, -463], [16337, 2323, -4895, -203, -2335], [-49925, -7260, 1347, 1809, -3483], [41978, 10876, 1546, -1946, -2124], [-38708, 6841, 6694, -2549, 2798], [-32261, -3694, -1055, 1983, 3489], [-28656, -4463, -1208, 2034, 1496], [36050, -1614, -1565, -3331, -1068], [-41206, 6746, -1268, -2572, 384], [-10427, -10145, -5189, -1952, 889], [21538, 2574, -1711, -1355, -554], [-5513, -16921, 1585, -2142, 3089], [-16105, -11183, 2006, -1123, 3233], [-83791, -5689, -278, 491, -463], [-65320, 2331, -5556, -1154, 2976], [-36581, -2952, 2298, -1238, 3529], [-2571, 741, 3758, -2760, 2957], [-51952, -2755, -1644, -4947, 2210], [37372, 6916, -364, 354, -420], [25715, 17102, -2629, 2647, -429], [39011, -12342, -1952, -1046, -899], [95773, 669, 1393, 941, 3909], [-81635, -4560, 1713, 639, -685], [20919, 6576, 2835, -1179, 4771], [9589, -22841, 1009, 1314, 639], [42054, -2975, -3541, 2076, -352], [-31833, -4962, 4407, 1281, 505], [-13239, 12355, 1949, 4230, -249], [-21900, 4547, 2071, -1742, 874], [-57405, 12215, 5386, -1089, 1645], [55229, -4376, 2250, 2926, -929], [-13832, -19866, 200, -474, 2226], [-53933, 3759, 431, 54, -3171], [-21430, -946, 974, -20, 2324], [439, 7905, 6592, -198, 811], [-87432, -4975, 2019, -243, -3116], [-30396, 15396, 5241, 1627, -208], [-37843, -929, 5071, -856, -1163], [2137, 13814, -2531, 1325, -2301], [-77354, 5380, -5454, -1914, 1072], [-10879, -5479, -3970, -1484, -4823], [96245, -5532, 2329, -229, 4175], [-1024, -3050, 517, 655, 3898], [-64008, 3457, -6084, -2976, -3072], [48451, -14396, -1110, 1806, 1025], [48153, 1157, -1163, 3665, -3879], [-89745, 12993, -1161, -1662, -95], [103633, -10565, -1979, 50, -1939], [46169, 6534, 5967, -2275, -1365], [4012, 10070, -6280, 2347, -1256], [105410, -7574, -1818, -1328, 3782], [-51407, 5727, 5389, 1010, -4039], [31093, -409, 3457, -2606, 4211], [-96465, 16240, 5590, 1374, 633], [-24326, 7640, 5458, -973, 114], [-48403, 18823, 5074, -1425, 3121], [-88430, 8468, -2811, 338, 2396], [-7349, 9693, -8212, -1652, -1413], [39608, 8244, -123, 7570, -1914], [-109854, -6718, 5211, -3673, 2100], [-47925, 19798, 2098, -436, 1305], [-40517, -7987, -482, -931, -313], [52281, 8697, 553, -1732, 6157], [-16484, 16662, 5975, 2133, 156], [20008, -6745, 6629, 2749, 1140], [19296, 1172, 1074, -1138, 6136], [-5912, -19943, 4229, -280, 3326], [-61832, -3870, 6, 2852, -5508], [-42060, -10861, 1291, -7397, 3112], [107753, 5970, 1920, 2070, 2339], [-63774, -10827, -3173, 3692, -3800], [12055, -13016, -3851, -747, -1803], [-82898, 3167, -1600, -2520, -2470], [-11059, 2580, -1615, 2339, -1689], [-79834, -1508, -1604, -6146, 2920], [12948, -8568, -556, 4930, 628], [17336, 14322, -3466, -1464, 1214], [41396, 2188, -4300, 722, 488], [-9383, -3549, -7927, -2292, -4419], [131915, 3049, -9655, 973, -669], [64889, 3140, -6101, 2653, -235], [8848, -3176, -6496, 6584, -5592], [80828, -1266, -554, 2084, -2590], [-20066, -3268, -1828, 6547, -3416], [-19001, 14999, 1235, 1644, -4579], [121130, -160, -10052, 1260, -2060], [87483, -8911, 1101, 4272, -1513], [93424, 11326, -1400, 2001, -2012], [33036, -14639, -416, 956, 2124], [96143, -7251, -394, 2858, -61], [112202, 9426, 6260, -4488, 3092], [-88554, 6754, -3964, -526, -3198], [71036, 2322, 3845, -5049, 4326], [70809, 937, -6963, -2062, 4769], [-466, 18023, -5939, -2370, -1376], [-5714, -14206, -3326, 2800, 2754], [76108, -1281, -1463, -1445, -1352], [-95490, 5588, 992, 1512, -2920], [-128660, 7270, 5244, -1222, 900], [18641, -4022, 7321, -1639, 2313], [34884, -1438, -7536, 3089, -1362], [19608, -13796, 818, 1340, 4596], [123959, -8759, -2674, 1875, -595], [-48338, -11248, -7160, 3100, 1768], [68186, -3851, -3245, 4448, -2288], [-56241, -1254, -9975, 131, -667], [-39440, 12900, 930, 240, 4862], [-88586, -9813, -5486, 414, 3874], [126121, 5961, -663, -3638, -4577]]
